<?php
/** @noinspection PhpFullyQualifiedNameUsageInspection */
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Doctrine\DBAL\Exception as DBALException;
use JsonException;
use RuntimeException;

use function add_action;
use function add_filter;
use function define;
use function defined;
use function file_exists;
use function file_get_contents;
use function json_decode;
use function maybe_serialize;
use function preg_match;
use function realpath;
use function sprintf;

/**
 * Class WordPressTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 *
 * @runTestsInSeparateProcesses
 */
abstract class WordPressTestCase extends ApplicationTestCase
{
    use DatabaseTestTrait;

    private array $providers = [
        \Inti\Db\ConfigProvider::class,
        \Inti\Wp\Middleware\ConfigProvider::class,
        \Inti\Wp\Plugin\ConfigProvider::class,
    ];

    private ?array $original_wp_options;
    private  array $added_options = [];
    private  array $updated_options = [];

    protected function setUp(): void
    {
        parent::setUp();
        $this->setUpWpOptions();
    }

    private function setUpWpOptions(): void
    {
        require_once ABSPATH . 'wp-includes/plugin.php';
        $this->original_wp_options = $this->wp_options ?? null;

        foreach (($this->wp_options ?? []) as $option => $value) {
            add_filter("pre_option_{$option}", fn() => $value);
        }

        add_action('add_option', fn($option) => ($this->added_options[] = $option));
        add_action('update_option', fn($option, $old_value) => ($this->updated_options[$option] = $old_value), 10, 2);
    }

    /**
     * @throws DBALException
     */
    protected function tearDown(): void
    {
        parent::tearDown();
        $this->tearDownWpOptions();
    }

    /**
     * @throws DBALException
     */
    protected function tearDownWpOptions(): void
    {
        if (isset($this->original_wp_options)) {
            /** @noinspection PhpFieldAssignmentTypeMismatchInspection */
            $this->wp_options = $this->original_wp_options;
        }

        $this->original_wp_options = null;

        $conn = $this->createConnection();

        foreach ($this->added_options as $option_name) {
            $conn->delete(
                'wp_options',
                ['option_name' => $option_name]
            );
        }

        foreach ($this->updated_options as $option_name => $option_value) {
            $conn->update(
                'wp_options',
                ['option_value' => maybe_serialize($option_value)],
                ['option_name' => $option_name]
            );
        }
    }

    /**
     * @throws JsonException
     */
    public static function setUpBeforeClass(): void
    {
        parent::setUpBeforeClass();
        self::defineABSPATH();
        self::checkWordPressIsInstalled();
    }

    /**
     * @throws JsonException
     */
    private static function defineABSPATH(): void
    {
        if (defined('ABSPATH')) {
            return;
        }

        $projectRoot = preg_match('#^(.+)/vendor/inti/test/src/TestCase$#', __DIR__, $matches)
            ? realpath($matches[1])
            : realpath(__DIR__ . '/../..');

        $composerJson = $projectRoot . '/composer.json';
        $composerJson = file_get_contents($composerJson);
        $composerJson = json_decode($composerJson, true, 512, JSON_THROW_ON_ERROR);

        $wordpressInstallDir = $composerJson['extra']['wordpress-install-dir'] ?? 'wordpress';
        $wordpressInstallDir = $projectRoot . '/' . $wordpressInstallDir;

        if (!is_dir($wordpressInstallDir)) {
            throw new RuntimeException(sprintf(
                "%s: WordPress installation directory '%s' does not exist; Maybe run 'composer install'?",
                __METHOD__, $wordpressInstallDir
            ));
        }

        if (!file_exists($wordpressInstallDir . '/wp-includes/plugin.php')) {
            throw new RuntimeException(sprintf(
                "%s: Directory '%s' does not contain a WordPress installation; Maybe run 'composer install'?",
                __METHOD__, $wordpressInstallDir
            ));
        }

        define('ABSPATH', $wordpressInstallDir . '/');
    }

    private static function checkWordPressIsInstalled(): void
    {
        $conn = (new static())->createConnection();

        try {
            $conn->executeQuery('SELECT 1 FROM wp_options');
            return;
        } catch (DBALException $e) {
            throw new RuntimeException(sprintf(
                "%s: WordPress not installed.",
                __METHOD__
            ));
        }
    }
}
